<!-- transformation sheet to eliminate living people from a GEDCOM file -->

<xsl:transform
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 version="1.0"
>

<xsl:param name="threshold" select="1920"/>

<!-- define keys to allow records to be found by their id -->

<xsl:key name="rec" match="INDI|FAM" use="@ID"/>

<xsl:template match="INDI" mode="exclude">
    <xsl:variable name="birth" select="substring(BIRT/DATE, string-length(BIRT/DATE - 3))"/>
    <xsl:choose>
    <xsl:when test="DEAT | BURI">no</xsl:when>
    <xsl:when test="$birth &lt; $threshold">no</xsl:when>
    <xsl:otherwise>yes</xsl:otherwise>
    </xsl:choose>
</xsl:template>

<xsl:template match="FAM" mode="exclude">
    <xsl:variable name="husband-exc">
        <xsl:apply-templates select="key('rec', HUSB/@REF)" mode="exclude"/>
    </xsl:variable>
    <xsl:variable name="wife-exc">
        <xsl:apply-templates select="key('rec', WIFE/@REF)" mode="exclude"/>
    </xsl:variable>     
    <xsl:choose>
    <xsl:when test="$husband-exc='no' or $wife-exc='no'">no</xsl:when>
    <xsl:otherwise>yes</xsl:otherwise>
    </xsl:choose>
</xsl:template>

<xsl:template match="INDI | FAM">
    <xsl:variable name="exclude">
        <xsl:apply-templates select="." mode="exclude"/>
    </xsl:variable>
    <xsl:choose>
    <xsl:when test="$exclude='no'">
        <xsl:copy>
            <xsl:copy-of select="@ID"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:when>
    <xsl:otherwise/>
    </xsl:choose>
</xsl:template>

<xsl:template match="*[@REF]">
    <xsl:variable name="exclude">
        <xsl:apply-templates mode="exclude" select="key('rec', @REF)"/>
    </xsl:variable>
    <xsl:choose>
    <xsl:when test="$exclude='no'">
        <xsl:copy>
            <xsl:copy-of select="@REF"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:when>
    <xsl:otherwise/>
    </xsl:choose>
</xsl:template>

<xsl:template match="*">
    <xsl:copy>
        <xsl:copy-of select="@ID"/>
        <xsl:copy-of select="@REF"/>
        <xsl:apply-templates/>
    </xsl:copy>
</xsl:template>

</xsl:transform>

