<xsl:transform
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   version="2.0"
   xmlns:xs="http://www.w3.org/2001/XMLSchema" 
   exclude-result-prefixes="xs">
   
<!-- Parse a GEDCOM 5.0 file --> 

<xsl:output method="xml" indent="yes"/>

<xsl:param name="input" as="xs:string" required="yes"/> 

<xsl:variable name="input-text" as="xs:string" 
                     select="unparsed-text($input, 'iso-8859-1')"/>

<xsl:variable name="lines" as="xs:string*" select="tokenize($input-text, '\r?\n')"/>

<xsl:variable name="parsed-lines">
<ged>
  <xsl:for-each select="$lines">
    <xsl:analyze-string select="." 
     regex="^([0-9]+)\s*(@([A-Za-z0-9]+)@)?\s*([A-Za-z]*)?\s*(@([A-Za-z0-9]+)@)?(.*)$"> 
      <xsl:matching-substring>
        <line level="{regex-group(1)}"
              ID="{regex-group(3)}"
              tag="{regex-group(4)}"
              REF="{regex-group(6)}"
              text="{regex-group(7)}"/>
      </xsl:matching-substring>
      <xsl:non-matching-substring>
        <xsl:message>Non-matching line "<xsl:value-of select="."/>"</xsl:message>
      </xsl:non-matching-substring>
    </xsl:analyze-string>
  </xsl:for-each>
</ged>
</xsl:variable>

<xsl:template name="process-level">
  <xsl:param name="population" required="yes" as="element()*"/>
  <xsl:param name="level" required="yes" as="xs:integer"/>
  <xsl:for-each-group select="$population" 
                                  group-starting-with="*[xs:integer(@level) eq $level]">
    <xsl:element name="{@tag}">
      <xsl:copy-of select="@ID[string(.)], @REF[string(.)]"/>
      <xsl:value-of select="normalize-space(@text)"/>
      <xsl:call-template name="process-level">
        <xsl:with-param name="population" select="current-group() except ."/>
        <xsl:with-param name="level" select="$level + 1"/>
      </xsl:call-template>
    </xsl:element>
  </xsl:for-each-group>
</xsl:template>

<xsl:variable name="nested">
<GED>
  <xsl:call-template name="process-level">
    <xsl:with-param name="population" select="$parsed-lines/ged/line"/>
    <xsl:with-param name="level" select="0"/>
  </xsl:call-template>
</GED>
</xsl:variable>

<xsl:template name="main">
   <xsl:copy-of select="$nested"/>
</xsl:template>

<xsl:template name="debug-parsed-lines">
  <xsl:copy-of select="$parsed-lines"/>
</xsl:template>

</xsl:transform>
